//+-------------------------------------------------------------------------------------------------+
//|																					!xMeterEA.mq4   |
//|																									|
//| 				This is an OpenSource effort collaborated by many people at TSD-Forum			|
//|																				Originator: FerruFx |
//|								Price Meter System GPL by Rlinac (TSD-forum) modified by jlust286 |
//|	Modicifations:																					|																		
//|		2007/05/16 Modified by Robert Hill IBFX:mini Account										|
//|		2008/10/20 Modified by jlust286 (Trading Rules)												|
//|		2010/08/26 Modified by Mikls Kiss (generalization to any pairs)							|
//|		2010/10/05 Modified by Mikls Kiss (trade logic made simpler)								|
//|     2010/10/27 Modified by Mikls Kiss (bugfix, CloseTrigger removed)							|
//|		2011/01/10 Modified by Guillermo Rexach (Complete rewrite)									|
//+-------------------------------------------------------------------------------------------------+

//EA SETUP:
// 1. Make sure using at least MT4 build 202.
// 2. Place the !xMeter.mq4 at experts folder.
// 3. Restart your terminal.
// 4. Show all symbols from Market watch by pressing Ctrl+M or from menu View->Market Watch.
// 5. Right click on Market Watch dialog then select Show All.
// 6. Attach EA to ONE chart.  Works on any chart, any timeframe, the results will be the same.
// 7. Select which currencies you want to trade using EA properties and check "Allow live trading".

//TRADING RULES:
// 1. Check if current time is at bPauseTrading otherwise trading is allowed.
// 2. Check EA properties if currency pair is allowed to be traded.
// 3. Enter a trade if the xMeter difference between the Major and Minor currencies > edTradeTrigger.
// 4. A trade may be closed 5 ways.  
//    - xMeter CloseTrigger (indicating reversal in short TF).
//    - Breakeven Stop Loss.
//    - Dynamic Trailing Stop.
//    - Dynamic Take Profit.
//    - Breakeven in Recovery Mode
// 5. If the currency's trades never reach Breakeven,they will be closed when the xMeter difference
//    between currencies <= to CloseTrigger using only the last 1/3 of lookback period.
// 6. If the trade becomes profitable,it will place a stop at Breakeven and trail with a DynamicStop.
// 7. If the difference between currencies falls to RecoverTrigger,a new trade will be opened in the
//    currency in the same direction and will open another if currency continues to go against us.
//    The trades will be closed when they average breakeven.
// 8. If the trade reaches Spread+StdDev*3 it is closed by Dynamic TakeProfit.

//HOW TO BACKTEST:
//		As of version 1.8 EA cannot be backtested. EA tries to meter Currencies against every other.
//		However MT4 only feeds 1 symbol's data while backtesting.
//		If you try to do a backtest, it won't execute any trades. Do forward test on a demo account.

//NOTES:
// 1.This tool can run on any platfrom regardless of your type of account. You might need to use the
//	 Suffix parameter, if broker symbol names have a common suffix(e.g.:EURUSDifx -> esSuffix="ifx")
// 2.The xMeter indicator gathers data from all available currency pairs. The indicator will be more
//   reliable if  you use a broker that offers more currencies. xMeter will search for all the known 
//	 currencies (112 as of 2010)  that the broker provides.
// 3.Please make any changes as needed.

//MODIFICATIONS, ADDITIONS, CHANGES, BUG FIXES:
//	Changes by jlust286 (www.stiglu.com) on 2008/10/20:
//		- Added MoneyManagment.
//		- Added bPauseTrading with default set to pause Friday and allow trades beginning Monday.
//		- Added edRecoverTrigger and CloseTrigger.
//		- Added BreakEven and DynamicStop.
//		- Added DynamicTakeProfit.
//		- Added Temporary SLTP feature to prevent massive account loss incase of internet disconnect
//		- Added grid averaging with increasing lots so new trades can be scaled in at better prices.
//		- Changed xMeter to lookback specified bars instead of Midnight of current day.
//		- Modified xMeter so last 3rd of eiLookBackPeriod is weighted 33% heavier than first 2/3rd
//		- Changed xMeter to use all available currency pairs to evaluate a currency's strength.
//		- Modified EA to be backtestable.
//	Changes by Mikls Kiss (http://mikki.hu) on 2010/08/26:
//		- Removed IBFX mini parameters
//		- Generalized pair setup:
//		- automatic pair detection for strength metering
//		- special symbols might be added (SpecialPairs)
//		- any pair can be used for trading (esTradePairs)
//		- added esSuffix parameter to handle any kind of brokers
//		- Added eiSpreadLimit variable instead of automatic spread limit detection
//		- Fixed stack overflow bugs
//		- Added support for 5 digit brokers
//		- Added NormalizeDouble() calls around prices (fix backtest issues)
//	Changes by Mikls Kiss (http://mikki.hu) on 2010/09/06:
//		- Currencies occurring in few pairs (<3) ignored
//		- Added automatic pair selection
//		- Added option to filter correlated pairs (done always in auto pair selection mode)
//		- Added option to trade in different directions(eiSwapMode:0=don't care;1=better;2=positive)
//		- Opening logic separated:
//		- recovery positions opened 1st. A pair will not be traded  unless previous position exists
//		- new positions are opened after (if margin allows) on allowed trade pairs
//		- Code fully revised (identation, minor bugs)
//	Changes by Mikls Kiss (http://mikki.hu) on 2010/09/06:
//		- parameters restructured
//		- added eiRecoveryPips parameter: in recovery mode aim for this many pips profit
//		- recovery TP set for breakeven level (+eiRecoveryPips pips), no separate position closing
//		- removed AdjustDynamicTPSL() function: not necessary any more
//		- removed ExitAllTradesNOW() function: not necessary any more
//		- code furter revised: less calculations, removed (some) unused variables
//	Changes by Mikls Kiss (http://mikki.hu) on 2010/10/27:
//		- CloseTrigger and order closing removed (grid averaging performs well)
//		- bug fixes related to suffixed brokers
//  Changes by Guillermo Rexach on 2011/01/10
//		- Additions:
//			replaced internal bubblesort with quickSort
//			replaced sequential searches (Currency) with binary searches
//			added Precision and Min Stops to Settings legend
//		- Changes:
//			--------------------------------------------------------------------------
//			Currency Order was reordered into the same sequence as original Indicator
//			--------------------------------------------------------------------------			
//			Duplicate code was replaced by common callable functions. This enabled a
//			source code reduction of over 7k(10%) and 4k reduction on object code. In
//			the process, a few bugs were discovered (sprdLimit being one)
//			--------------------------------------------------------------------------
//			The Buy, Sell and Symbol Arrays were moved to the Global Area.
//			--------------------------------------------------------------------------
//		- Additional Notes
//			--------------------------------------------------------------------------
//			1.Amounts displayed match EXACTLY with version !xMeter_v2.1. The amounts
//			  vary slightly with version !xMeter_v2.1.1, a later release.
//			2.Lists of Allowed pairs match EXACTLY with version !xMeter_v2.1. The List
//			  varies considerably with version !xMeter_v2.1.1, a later release.
//			--------------------------------------------------------------------------
//			Naming Conventions were improved by the use of prefixes for the most part.
//			--------------------------------------------------------------------------
//			Primitive Debugger added to log function execution sequence. To Activate,
//          replace all //DeBug & //DeBugEnd with Debug & DeBugEnd, & set eiDeBug=true
//			--------------------------------------------------------------------------
//			pairsRelated Function used to filter pairs that appear on the correlated
//			list but are not associated with either the Base or the Quote Currency.
//			(To disable, uncomment 1st line in the pairsRelated Function.)
//			--------------------------------------------------------------------------
//===================================================================================================
//===================================================================================================
//        quickSort(string& stringArray[],int beginningElementNumber=0, int numberOfElements-1)
#include <quickSort.mqh>
#define	SCORELEVELS								10
#define SWAPDONTCARE							0		// don't care with swap
#define SWAPBETTER								1		// trade pair in better swap direction only
#define SWAPPOSITIVE							2		// trade pair in non-negative swap direction
#define MINIMUMCURRENCYPAIRS					3		// minimum number of pairs currencies must
														// occur in to be tradeable

#define NL										"\n"	// NewLine Character
#define CORRELATION								0.75	// correlation limit

//---- Trade parameters
extern	bool		ebDeBug				=		false;
extern	string		esEAversion      	=		"!xMeterEA.0";
extern	string		label_0             =		"=== Trade parameters ===";
extern	int			eiMaxUsedMargin		=		00010;	// maximum used margin to open a trade 
														// (Won't open new if used margin is > this)
extern	double		edAccountRiskPercent=		00.10;
extern	double		edMaxInitialLotSize	=		03.20;	// maximum lot size for non-recovery orders
extern	int			eiSlippage			=			2;	// Slippage
extern	int			eiMagicNo			=		821962;	// Magic Number
extern	int			eiSwapMode			=SWAPDONTCARE; 	// 0: don't care; 
														// 1: only in 'better' direction;
														// 2: only positive
extern	int			eiPauseStartTime	=		60000;	// bPauseTrading DayofWeek,Hour,Minute.
														// Example input: 50700 will pause trading
														// on friday at 7:00 am.  Current trades will
														// continue until CloseTrigger reached or
														// stopped out. Set to 0 to disable.
extern	int			eiPauseEndTime		=		00000;	// End pause trading.
														// Example input: 10800 will allow trading 
														// after 8:00am on monday.
extern	int			eiBreakEvenPips		= 			3;	// breakeven level(breakeven + BreakEvenPips)
extern	int			eiRecoveryPips		=			3;	// recovery exit at this many pips profit
														// (relative to average price)
extern	bool		ebAllowNewTrades	=		 true;	// allow opening of new trades(doesnt affect
														// recovery positions)

extern	string		label_1             = "=== Pairs Selection ===";
extern	string		esSuffix			=   		"";	// common(broker specific)suffix for symbols
														// (e.g.: "m", "ifx", etc.)
extern	bool		ebAutoSelectPairs	=         true; // if true,EA automatically chooses pairs to
														// trade in the given circumstances
extern	string		esSpecialPairs      =           ""; // list here all other pairs that should be 
														// watched,but AutoSelectPairs doesn't find 
														// because of special name
extern	string		esTradePairs		=           ""; // comma separated list of pairs to trade on
														// (used only if ebAutoSelectPairs is false)
extern	int			eiSpreadLimit		=           35; // do not trade pair if it's current spread 
														// is > eiSpreadLimit
extern	bool		ebCorrelationFilter	=         true; // use correlation filtering - do not open 
														// 'similar' trades of correlating pairs

extern	string		label_2				= "=== Signals ===";
extern	int			eiLookBackPeriod	=			84;	// How many hours to look back to calculate 
														// High and Low for xMeter?
extern	double		edTradeTrigger		=		   3.5;	// Value must be greater than RecoverTrigger
														// and less than 9.  Will enter trade when 
														// currencies xMeter difference is greater
														// than edTradeTrigger.
extern	double		edRecoverTrigger	=	   		00;	// Will allow another trade in same currency 
														// if xMeter falls below edRecoverTrigger,
														//then goes back up to edTradeTrigger.
extern	int			eiBlockSize			=			10;	//	Number of Elements to resize
//===================================================================================================
//===================================================================================================

//+**************************************************************************************************
//+-- Open position Data ---																		|
//+**************************************************************************************************
		int			iaCount[];							// Total number of Open positions
	
		int			iPairType;							// Number of Different open positions	
		string		saCorrel[],							// list of correlating symbols
					saPairType[];						// symbol name and direction

		double		daPrice[],							// average price
					daLots[],     						// sum of lots
					daProfit[],							// total profit (in deposit currency)
					daFees[];							// swap and commission

//+**************************************************************************************************

		int			iPairBuy;							// number of pairs allowed	(buy)
		string		saPairBuy[];						// pairs usable for trade	(buy)

		int			iPairSell;							// number of pairs allowed	(sell)
		string		saPairSell[];						// pairs usable for trade	(sell)

		int			iPair;								// number of pairs
		string		saPair[];							// pairs watched(EURUSD,GBPJPY,etc.)

		int			iCurrency;							// number of Currencies
		string		saCurrency[];						// Currency Name(EUR,USD,JPY,CHF,etc.)
		
		int
					iaDivisor[],						// Number of Currencies for each pair
					iaPairCount[];						// Number of Pairs involving each Currency

		int			iBuy;
		string		saBuy[];

		int			iSell;
		string		saSell[];

		int			iCount;
		string		saSymbols[];		

		double		
					daTriggerMeter[],
					daTriggerHigh[],
					daTriggerLow[],
					daTriggerAsk[],
					daTriggerRatio[],
					daTriggerRange[],
					daTriggerMajor[],
					daTriggerMinor[],

					daCurrencyMeter[],
					daCurrencyHigh[],
					daCurrencyLow[],
					daCurrencyAsk[],
					daCurrencyRatio[],
					daCurrencyRange[],
					daCurrencyMajor[],
					daCurrencyMinor[],
					daCurrencyWeight[];

		string
					sIndent,
					sMessage,
					sSymbol,
					sPauseTime,
					sPairType,
					sRoot,
					saOrderType[]		=	{	"(+)",	"(-)"	},
					saSwapText[]		=	{
												"don't care",
												"better direction",
												"non-negative direction"
											};

		double
					dStopLoss,
					dTakeProfit,
					dInitialLotSize;
					
		int			
					iTick,
					iCycle,
					iIndex,
					iLotPrecision,	
					iSortTimes,
					iaGrade[]			=	{	00,03,10,25,40,50,60,75,90,97	};


		datetime	
					tNextPairSelectionTime,
					tPrintErrorTime,
					tLastErrorTime;

		bool
					bPauseTrading,
					bInitialized,
					bAlertNow;
					
		int			iBase,
					iQuote,
					iDigit;


void DeBug(string sName)
	{
		if (ebDeBug)
			{
				Print(sIndent + sName);
				sIndent	=	sIndent + "   ";
				if (StringLen(sIndent)> 96)
					sIndent	=	"   ";
			}
	}
void DeBugEnd()
	{
		if (ebDeBug)
			sIndent	=	StringSubstr(sIndent,3);
	}
//===================================================================================================
//+-------------------------------------------------------------------------------------------------+
//| expert initialization function                                   								|
//+-------------------------------------------------------------------------------------------------+
int init() 
	{
	//DeBug("init");
		initializeExpert();
	//DeBugEnd();
	}

void initializeExpert()
	{
	//DeBug("initializeExpert");
		checkAlert();
		//--- construct list of possible currencies ---
		string	CURRENCIES[]		=	{
											"AFN","ALL","ANG","ARS","AUD","AWG","AZN","BAM",
											"BBD","BGN","BMD","BND","BOB","BRL","BSD","BWP",
											"BYR","BZD","CAD","CHF","CLP","CNY","COP","CRC",
											"CUP","CZK","DKK","DOP","EEK","EGP","EUR","FJD",
											"FKP","GBP","GGP","GHC","GIP","GTQ","GYD","HKD",
											"HNL","HRK","HUF","IDR","ILS","IMP","INR","IRR",
											"ISK","JEP","JMD","JPY","KGS","KHR","KPW","KYD",
											"KZT","LAK","LBP","LKR","LRD","LTL","LVL","MKD",
											"MNT","MUR","MXN","MYR","MZN","NAD","NGN","NIO",
											"NOK","NPR","NZD","OMR","PAB","PEN","PHP","PKR",
											"PLN","PYG","QAR","RON","RSD","RUB","SAR","SBD",
											"SCR","SEK","SGD","SHP","SOS","SRD","SVC","SYP",
											"THB","TRL","TRY","TTD","TVD","TWD","UAH","USD",
											"UYU","UZS","VEF","VND","XCD","YER","ZAR","ZWD"
										};
		constructBrokerPairs(CURRENCIES);
		Comment("Initializing EA, waiting for ticks...");
		constructCurrencies();
		allocateArrays();

		double	d	=	1;	//	Calculate allowed lot precision	
		for(;iLotPrecision<3&&MarketInfo(Symbol(),MODE_LOTSTEP)<d;iLotPrecision++)
			d/=10;

		iTick	=	Volume[0];
		
		MathSrand(TimeLocal());
		iCycle	=	MathRand();
		sRoot	=	"!xMeterEA"+iCycle%9+".";
		GlobalVariableSet(sRoot+".Initialized",iCycle);
 		sPauseTime=PauseToStr(eiPauseStartTime)+" .. "+PauseToStr(eiPauseEndTime);
	//DeBugEnd();
	}

//+-------------------------------------------------------------------------------------------------+
//| expert deinitialization function																|
//+-------------------------------------------------------------------------------------------------+
int deinit() 
	{
	//DeBug("deinit");
		GlobalVariablesDeleteAll(sRoot);
	//DeBugEnd();
		return(0);
	}

//+-------------------------------------------------------------------------------------------------+
//| expert start function																			|
//+-------------------------------------------------------------------------------------------------+
int start() 
	{
	//DeBug("start");sIndent="";
		if (!IsTesting())
			checkForMultipleExecutions();
		// give xMeter 3 ticks from start to gather all currency data before allowing trade.
		bInitialized	=	(iTick+3<Volume[0] || IsTesting());
		dInitialLotSize	=	autoLot();
		pauseAtTime(eiPauseStartTime,eiPauseEndTime);
		
		collectExistingTrades();
		calculateAveragePrices();
		calculateMeter();
		calculateDivisors();
		calculateAllCurrencies();

		processOpenClosePositions();
		//--- open new positions ---
		if (TimeLocal()	>	tNextPairSelectionTime)
			{
				tNextPairSelectionTime = TimeLocal() + 60; //recalculate every minute
				checkTradeList(ebCorrelationFilter);
			}

		if (ebAllowNewTrades)
			setUpNewTrades();
			
		displayInformation();
	//DeBugEnd();
	} 

double calculatedScore(double pdRatio)
	{
	//DeBug("calculatedScore");//DeBugEnd();
		int	i;
		while(i	<	SCORELEVELS	&&	pdRatio*100	>=	iaGrade[i])
			i++;
		if (i	>	1)
			return(i-1);
		return(0.001);
	}

//+-------------------------------------------------------------------------------------------------+
//| Open new buy order (recovery if possible or normal)												|
//| Returns true if order successfully opened false otherwise										|
//+-------------------------------------------------------------------------------------------------+
bool manageBuy(string sSymbol,int digit,string meters) 
	{
	//DeBug("manageBuy<"+sSymbol+">");
		double ask		=	MarketInfo(sSymbol, MODE_ASK);
		double point	=	MarketInfo(sSymbol, MODE_POINT);
		double sprd		=	MarketInfo(sSymbol, MODE_SPREAD);
		double sprdLm	=	eiSpreadLimit;
		if (digit	==	3	||	digit	==	5) 
			{
				point	*=	10;
				sprdLm	*=	10;
			}
		if (!bInitialized)
			return(false);
		if (sprd	<=	sprdLm) 
			{
				int		BuyOrders,
						SellOrders,
						BuyProfits;
						
				double	lt	=	dInitialLotSize,
						LowestBuy	=	1000;
				int	i,j	=	OrdersTotal();
				for(;i	<	j;i++) 
					if (OrderSelect(i,SELECT_BY_POS,MODE_TRADES)
					&&	OrderMagicNumber()	==	eiMagicNo
					&&	OrderSymbol()		==	sSymbol)
						if (OrderType()		==	OP_SELL)
							SellOrders ++;
						else
							if (OrderType()	==	OP_BUY)
								{
									BuyOrders++;
									if (OrderOpenPrice()<LowestBuy)
										LowestBuy=OrderOpenPrice();
								}							
				//Set minimum of 20 for grid spacing
				double	sdv=MathMax(point*20,
									point*sprd*3+MathPow(iStdDev(sSymbol,15,6,0,0,0,0),0.9));
				if (BuyOrders	==	0	&&	!bPauseTrading
				&&	GlobalVariableGet(sRoot+sSymbol+"buy")==1) 
					//Do not open new iCycle if current trade margin >	than allowed margin.
					if ((1 -(AccountFreeMarginCheck(sSymbol,OP_BUY,lt)/
							(AccountBalance()+AccountCredit())))*100	>	eiMaxUsedMargin)
						return(false);
					else
						lt=dInitialLotSize;
				else
					if (BuyOrders	>	0	&&	ask	<=	LowestBuy-sdv 
					&&	GlobalVariableGet(sRoot+sSymbol+"buy")==2) 
						//---- we can go trading
						if (BuyOrders	>	3)
							lt=NormalizeDouble(MathPow(	1.5,BuyOrders-2)*
														dInitialLotSize,iLotPrecision);
						else
							lt=NormalizeDouble(dInitialLotSize,iLotPrecision);
					else
						return(false);
				BuyOrders++;
				int		Retry,
						ticket;
				while(Retry	<	5	&&	!IsTradeAllowed()) 
					{
						Retry++;
						Sleep(2000);
					}
				ticket	=	OrderSend(	sSymbol,OP_BUY,lt,
										NormalizeDouble(ask,digit),
										eiSlippage,0,0,
										esEAversion+" #"+BuyOrders+" "+meters,eiMagicNo,0,Blue);
				Sleep(1000);
				if (ticket	>	0) 
					{
						if (OrderSelect(ticket,SELECT_BY_TICKET,MODE_TRADES)) 
							{
								GlobalVariableSet(sRoot+sSymbol+"buy",2);
								Print("BUY order opened: ",sSymbol," at ",OrderOpenPrice());
								return(true);
							}
					}
				else
					Print("Error opening ",sSymbol," BUY order - Error#: ",GetLastError());
			}
		else
			Print("Error opening ",sSymbol," BUY order - Error#: BigSpread");
		return(false);
	//DeBugEnd();
	}

//+-------------------------------------------------------------------------------------------------+
//| Open new sell order (recovery if possible or normal)											|
//| Returns true if order successfully opened false otherwise										|
//+-------------------------------------------------------------------------------------------------+
bool manageSell(string sSymbol,int digit,string meters) 
	{
	//DeBug("manageSell<"+sSymbol+">");
		double bid		=	MarketInfo(sSymbol, MODE_BID);
		double point	=	MarketInfo(sSymbol, MODE_POINT);
		double sprd		=	MarketInfo(sSymbol, MODE_SPREAD);
		double sprdLm	=	eiSpreadLimit;
		if (digit	==	3	||	digit	==	5) 
			{
				point	*=	10;
				sprdLm	*=	10;
			}
		if (!bInitialized)
			return(false);
		if (sprd<=sprdLm) 
			{
				int		BuyOrders=0,
						SellOrders=0,
						SellProfits=0;
						
				double	lt	=	dInitialLotSize,
						HighestSell	=	0;

				int	i,j	=	OrdersTotal();		
				for(;i	<	j;i++) 
					if (OrderSelect(i,SELECT_BY_POS,MODE_TRADES)
					&&	OrderMagicNumber()	==	eiMagicNo
					&&	OrderSymbol()		==	sSymbol
					&&	OrderType()			<=	OP_SELL)
						if (OrderType()		==	OP_BUY)
							BuyOrders++;
						else
							{
								SellOrders++;
								if (OrderOpenPrice()>HighestSell)
									HighestSell=OrderOpenPrice();
							}
				//Set minimum of 20 for grid spacing
				double	sdv	=	MathMax(point*20,
										point*sprd*3+MathPow(iStdDev(sSymbol,15,6,0,0,0,0),0.9));
				if (SellOrders	==	0	&&	!bPauseTrading
				&&	GlobalVariableGet(sRoot+sSymbol+"sell")==1) 
					//	we can go trading
					//  Do not open new iCycle of trades if current trades
					//  are using more than allowed margin.
					if ((1 -(AccountFreeMarginCheck(sSymbol,OP_SELL,lt)/(AccountBalance()+
							 AccountCredit())))*100>eiMaxUsedMargin)
						return(false);
					else
						lt	=	dInitialLotSize;
				else
					if (SellOrders	>	0	&&	bid	>=	HighestSell+sdv
					&&	GlobalVariableGet(sRoot+sSymbol+"sell")==2) 
						if (SellOrders>3)
							lt	=	NormalizeDouble(MathPow(1.5,SellOrders-2)*															dInitialLotSize,iLotPrecision);
						else
							lt	=	NormalizeDouble(dInitialLotSize,iLotPrecision);
					else
						return(false);
				SellOrders++;
				int	Retry=0,
					ticket=0;
				while(Retry	<	5	&&	!IsTradeAllowed()) 
					{
						Retry++;
						Sleep(2000);
					}
				ticket=OrderSend(sSymbol,	OP_SELL,lt,NormalizeDouble(bid,digit),eiSlippage,0,0,
											esEAversion+" #"+SellOrders+" "+meters,eiMagicNo,0,Red);
				Sleep(1000);
				if (ticket>0) 
					{
						if (OrderSelect(ticket,SELECT_BY_TICKET,MODE_TRADES)) 
							{
								GlobalVariableSet(sRoot+sSymbol+"sell",2);
								Print("SELL order opened : ",sSymbol," at ",OrderOpenPrice());
								return(true);
							}
					}
				else
					Print("Error opening ",sSymbol," SELL order : ",GetLastError());
			}
		else
			Print("Error opening ",sSymbol," SELL order - Error#: BigSpread");
		return(false);
	//DeBugEnd();
	}

//+-------------------------------------------------------------------------------------------------+
//| If trade is in profit > than breakeven then set breakeven										|
//| stop or adjust DynamicStop.																		|
//+-------------------------------------------------------------------------------------------------+
void dynamicBuySL(string sSymbol,int BuyOrders,double bid,double point,int sprd,int digit) 
	{
	//DeBug("dynamicBuySL");
		//Use Spread+StdDev to set SL 
		double	sdv			=	MathMax	(point*25, 		
										(point*sprd*3)+iStdDev(sSymbol,15,6,0,0,0,0)*1.5);
		double	Breakeven	=	sdv/1.5;
		int	i,j	=	OrdersTotal();
		for(;i	<	j;i++) 
			if (OrderSelect(i,SELECT_BY_POS,MODE_TRADES)
			&&	OrderMagicNumber()	==	eiMagicNo
			&&	OrderSymbol()		==	sSymbol
			&&	OrderType()			==	OP_BUY) 
				{
					if (BuyOrders==1) 
						{
							if (bid-OrderOpenPrice()+eiBreakEvenPips*point>=Breakeven
							&&	OrderStopLoss()<OrderOpenPrice())
								OrderModify(	OrderTicket(),
												OrderOpenPrice(),
												NormalizeDouble(OrderOpenPrice()+
																eiBreakEvenPips*point,digit),
												OrderTakeProfit(),0,Red);
							dStopLoss	=	NormalizeDouble(bid-sdv,digit);
							if (dStopLoss>0 && dStopLoss-OrderOpenPrice()>0 
							&&	dStopLoss-OrderStopLoss()>5*point) 
								OrderModify(	OrderTicket(),			// adj trailstop in 5 pip increments
												OrderOpenPrice(),
												NormalizeDouble(dStopLoss,digit),
												OrderTakeProfit(),0,Red);
						}
					if (OrderStopLoss()==0 || OrderStopLoss()<OrderOpenPrice()) 
						{
							dStopLoss=NormalizeDouble(bid-200*point,digit);
							if (MathAbs(OrderStopLoss()-dStopLoss)>10*point)
								OrderModify(	OrderTicket(),
												OrderOpenPrice(),
												dStopLoss,
												OrderTakeProfit(),0,Red);
						}
				}
	//DeBugEnd();
	}

//+------------------------------------------------------------------+
//| If trade is in profit greater than breakeven then set breakeven  |
//| stop or adjust DynamicStop.                                      |
//+------------------------------------------------------------------+
void dynamicSellSL(string sSymbol,int SellOrders,double ask,double point,int sprd,int digit) 
	{
	//DeBug("dynamicSellSL");
		//Set minimum of 25 for trailing stop
		double	sdv	=	MathMax	(point*25, 	//Use Spread+StdDev to set SL 
								(point*sprd*3)+iStdDev(sSymbol,15,6,0,0,0,0)*1.5);
		double	Breakeven	=	sdv/1.5;

		int	i,j	=	OrdersTotal();
		for(;i	<	j;i++) 
			if (OrderSelect(i,SELECT_BY_POS,MODE_TRADES)
			&&	OrderMagicNumber()	==	eiMagicNo
			&&	OrderSymbol()		==	sSymbol
			&&	OrderType()			==	OP_SELL)
				{
					if (SellOrders	==	1) 
						{
							if (OrderOpenPrice()-eiBreakEvenPips*point-ask>=Breakeven
							&& (OrderStopLoss()>OrderOpenPrice() || OrderStopLoss()==0))
								OrderModify(	OrderTicket(),
												OrderOpenPrice(),
												NormalizeDouble(OrderOpenPrice()	-
																eiBreakEvenPips*point,digit),
												OrderTakeProfit(),0,Red);
							dStopLoss=NormalizeDouble(ask+sdv,digit);
							if (dStopLoss>0 && OrderOpenPrice()-dStopLoss>0 
							&&	OrderStopLoss()-dStopLoss>5*point)
								//Adjust in 5 pip increments
								OrderModify(	OrderTicket(),
												OrderOpenPrice(),
												NormalizeDouble(dStopLoss,digit),
												OrderTakeProfit(),0,Red);
						}
					dStopLoss	=	NormalizeDouble(ask+200*point,digit);
					if((OrderStopLoss()	==	0	||	OrderStopLoss()	>	OrderOpenPrice())
					&&	MathAbs(OrderStopLoss()-dStopLoss)>10*point)
						OrderModify(	OrderTicket(),
										OrderOpenPrice(),
										dTakeProfit,
										OrderTakeProfit(),0,Red);
				}
	//DeBugEnd();
	}

//+-------------------------------------------------------------------------------------------------+
//| Adjust takeprofit as three times the Standard Deviation          								|
//+-------------------------------------------------------------------------------------------------+
void dynamicBuyTP(string sSymbol,int BuyOrders,double point,int sprd,int digit,double avgprice) 
	{
	//DeBug("dynamicBuyTP");
		int	i,j	=	OrdersTotal();
		if (BuyOrders	==	1) 
			for(;i	<	j;i++) 
				if (OrderSelect(i,SELECT_BY_POS,MODE_TRADES)
				&&	OrderMagicNumber()	==	eiMagicNo
				&&	OrderSymbol()		==	sSymbol
				&&	OrderType()			==	OP_BUY) 
					{	
						//Use Spread+StdDev to set TP				
						double sdv	=	MathMax	(point*30,
											(sprd*point*3)+iStdDev(OrderSymbol(),15,6,0,0,0,0)*3); 
						dTakeProfit	=	OrderOpenPrice()+sdv;
						if (MathAbs(OrderTakeProfit()-dTakeProfit)>5*point) // Adjust 5PiP Steps
							OrderModify(	OrderTicket(),
											OrderOpenPrice(),
											OrderStopLoss(),
											NormalizeDouble(dTakeProfit,digit),0,Blue);
					}
		else
			if (BuyOrders	>	1) 
				{
					avgprice	=	NormalizeDouble(MathCeil(avgprice/Point)*Point+eiRecoveryPips*point,
														digit);
					for(;i	<	j;i++) 
						if (OrderSelect(i,SELECT_BY_POS,MODE_TRADES)
						&&	OrderMagicNumber()	==	eiMagicNo
						&&	OrderSymbol()		==	sSymbol
						&&	OrderType()			==	OP_BUY
						&&	OrderTakeProfit()	!=	avgprice)
							OrderModify(	OrderTicket(),
											OrderOpenPrice(),
											OrderStopLoss(),
											avgprice,0,CLR_NONE);
				}
	//DeBugEnd();
	}
	
//+-------------------------------------------------------------------------------------------------+
//| Adjust takeprofit as three times the Standard Deviation          								|
//+-------------------------------------------------------------------------------------------------+
void dynamicSellTP(string sSymbol,int SellOrders,double point,int sprd,int digit,double avgprice) 
	{
	//DeBug("dynamicSellTP");
		int	i,j	=	OrdersTotal();
		if (SellOrders	==	1) 
			for(;i	<	j;i++) 
				if (OrderSelect(i,SELECT_BY_POS,MODE_TRADES)
				&&	OrderMagicNumber()	==	eiMagicNo
				&&	OrderSymbol()		==	sSymbol
				&&	OrderType()			==	OP_SELL) 
					{
						//Use Spread+StdDev to set TP
						double sdv	=	MathMax	(point*30,
												(point*sprd*3)+iStdDev(OrderSymbol(),15,6,0,0,0,0)*3);
						dTakeProfit=OrderOpenPrice()-sdv;
						if (MathAbs(OrderTakeProfit()-dTakeProfit)>5*point) //Adj in 5pip increments
							OrderModify(	OrderTicket(),
											OrderOpenPrice(),
											OrderStopLoss(),
											NormalizeDouble(dTakeProfit,digit),0,Blue);
					}
		else
			if (SellOrders	>	1) 
				for(;i	<	j;i++) 
					if (OrderSelect(i,SELECT_BY_POS,MODE_TRADES)
					&&	OrderMagicNumber()	==	eiMagicNo
					&&	OrderSymbol()		==	sSymbol
					&&	OrderType()			==	OP_SELL
					&&	OrderTakeProfit()	!=	avgprice)
						OrderModify(	OrderTicket(),
										OrderOpenPrice(),
										OrderStopLoss(),
										NormalizeDouble(MathFloor(avgprice/Point)*Point-
														eiRecoveryPips*point,digit),
										0,CLR_NONE);
	//DeBugEnd();
	}

//+-------------------------------------------------------------------------------------------------+
//| Will pause trading between startime and endtime.												|
//+-------------------------------------------------------------------------------------------------+
int pauseAtTime(int startTime,int endTime) 
	{
	//DeBug("pauseAtTime");
		if (startTime!=0) 
			{
				int t = DayOfWeek() * 10000 + TimeHour(TimeCurrent()) * 100 + TimeMinute(TimeCurrent());
					bPauseTrading=(t >= startTime || t <= endTime);
				//Allow current iCycle of trades to continue until all are closed by EA then pause
			}
	//DeBugEnd();
	}
	
//+-------------------------------------------------------------------------------------------------+
//| Calculate lot size based on account balance and risk settings									|
//+-------------------------------------------------------------------------------------------------+
double autoLot() 
	{
	//DeBug("autoLot");//DeBugEnd();
		double lot	=	MathMin(NormalizeDouble((	AccountBalance()+AccountCredit())/10000*
													edAccountRiskPercent,iLotPrecision),
													edMaxInitialLotSize);
		return(MathMax(lot,MarketInfo(Symbol(),MODE_MINLOT)));
	}

//+-------------------------------------------------------------------------------------------------+
//| Display information                                              								|
//+-------------------------------------------------------------------------------------------------+
void displayInformation() 
	{
	//DeBug("displayInformation");
		//--- account properties ---
		sMessage	=	NL	+	"Account properties:"	+
						NL	+	"   Leverage                 : "	+	AccountLeverage()	+	":1"	+
						NL	+	"   Lot (min / max / step ): "		+	
											DoubleToStr(MarketInfo(Symbol(),MODE_MINLOT),2)
							+	" / "	+	DoubleToStr(MarketInfo(Symbol(),MODE_MAXLOT),2)
							+	" / "	+	DoubleToStr(MarketInfo(Symbol(),MODE_LOTSTEP),2)	+ NL +
						NL	+	"Settings:"	+
						NL	+	"   AccountRisk     : "	+	DoubleToStr(edAccountRiskPercent,4)
							+	" ("					+	DoubleToStr(autoLot(),2)	
							+	" lots; limit: "		+	DoubleToStr(edMaxInitialLotSize,2)
							+	" lots)"	+
						NL	+	"   SwapMode      : "	+	eiSwapMode+" ("+saSwapText[eiSwapMode]+")"+ 
						NL	+	"   SpreadLimit      : "+	eiSpreadLimit	+	" ("+ Digits + " digit Broker detected)" 
							+	"   Precision: " +	iLotPrecision	
							+	", MinStop: "	+	DoubleToStr(MarketInfo(Symbol(),MODE_STOPLEVEL)/10,1)+	
						NL	+	"   Weekend pause: "	+	sPauseTime	+	NL	+	
						NL	+	"Currency strength: ("	+	TimeToStr(TimeCurrent(),TIME_DATE)	 +	" "
												+	TimeToStr(TimeCurrent(),TIME_SECONDS)+	")"	+	NL;
		string	s[]	=   {"","","","","","",""},
				m[]	=   {"","","","","","",""},
				c[]	=   {"","","","","","",""};
		int		i,z;
		for(;i	<	iCurrency;i++)
			if (iaPairCount[i]	>=	MINIMUMCURRENCYPAIRS)
				{
					s[z]	=   currencyStrength(i);
					m[z]	=   DoubleToStr(daCurrencyMeter[i],1);
					c[z]	=   saCurrency[i];
					z++;
				}
		for(z=0;z	<	7;z++)
			sMessage		=	sMessage	+	"   ["	+	s[seq(z)]
											+	"]  "	+	m[seq(z)]
											+	"  "	+	c[seq(z)]	+	NL;

		sMessage	=	sMessage	+	NL	+
						"Allowed pairs for new orders & directions in priority order:"+NL;
		if (ebAllowNewTrades) 
			{
				sMessage=sMessage+"   buy: ";
				int count	=	0,
					spread	=	0;
				for(i=0;i	<	iPairBuy;i++) 
					{
						spread	=	MarketInfo(saPairBuy[i],MODE_SPREAD);
						if (spread	<=	eiSpreadLimit) 
							{
								if (count>0) 
									{
										sMessage=sMessage+"; ";
										if (count%10==0) 
											sMessage	=	sMessage	+	NL	+	"           ";
									}
								sMessage	=	sMessage	+	StringSubstr(saPairBuy[i],0,6);
								count++;
							}
					}
				sMessage	=	sMessage	+	NL	+	"   sell : ";
				count 		=	0;
				for(i=0;i	<	iPairSell;i++) 
					if (MarketInfo(saPairSell[i],MODE_SPREAD)	<=	eiSpreadLimit) 
						{
							if (count	>	0) 
								{
									sMessage	=	sMessage+"; ";
									if (count%10	==	0)
										sMessage	=	sMessage+NL+"           ";
								}
							sMessage	=	sMessage+StringSubstr(saPairSell[i],0,6);
							count++;
						}
			}
		else
			sMessage	=	sMessage+"   No new trades are allowed (ebAllowNewTrades = FALSE) "
									+	"except for recovery positions!";

		//--- active trades ---
		sMessage	=	sMessage+NL+NL+"Working pairs (used margin: "+
						DoubleToStr((1-AccountFreeMargin()/(AccountBalance()+
						AccountCredit()))*100,1)+"%; limit: "+eiMaxUsedMargin+"%):"+NL;
		if (iPairType	>	0) 
			for(i=0;i	<	iPairType; i++) 	
				{
					sMessage=sMessage+"   "+
					iaCount[i]+" "+StringSubstr(saPairType[i],0,6)+
					typeCode(saPairType[i])+
					": "+leftPad(DoubleToStr(daProfit[i],2),6,"  ");	
					if (i	<	ArraySize(saCorrel))
						if (saCorrel[i]	!=	"")
							sMessage=sMessage+"   correlated pairs: "+saCorrel[i]+NL;
						else
							sMessage=sMessage+NL;
					else
						sMessage=sMessage+NL;
				}
		else
			sMessage=sMessage+"   (none)";
		Comment(sMessage);
	//DeBugEnd();
	}

string typeCode(string sPairType)
	{
		if (StringFind(sPairType,"-")	<	0)
			return("-Buy ");
		return("-Sell ");
	}

//+-------------------------------------------------------------------------------------------------+
//| Return currency strength as a string															|
//+-------------------------------------------------------------------------------------------------+
string currencyStrength(int iIndex) 
	{
	//DeBug("currencyStrength");//DeBugEnd();
		int	value	=	MathMin(18,MathRound(2.0*daCurrencyMeter[iIndex]-0.5));
		switch(value) 
			{
  				case 00:	return(	"                                    ");
				case 18:	return(	"i!i!i!i!i!i!i!i!i!i!i!i!i!i!i!i!i!i!i!i!i!i!i!i!i!i!i!");
				default:	return(	StringSubstr(
									"i!i!i!i!i!i!i!i!i!i!i!i!i!i!i!i!i!i!i!i!i!i!i!i!i!i!i!", 
									0, MathRound(value * 3)) +
                      				StringSubstr("                                    ",
									0,36-MathRound(2*value)));
			}
	}

//+-------------------------------------------------------------------------------------------------+
//| calculate correlation between the two symbols													|
//+-------------------------------------------------------------------------------------------------+
double Correlation(string Symbol1,string Symbol2, int i=0) 
	{
	//DeBug("Correlation"+ "Pair1: "+ Symbol1+", Pair2: "+Symbol2);

		double	sum_xy = 0,
				sum_x  = 0,
				sum_y  = 0,
				sum_xx = 0,
				sum_yy = 0;

		for(;i	< eiLookBackPeriod;i++) 
			{
				double x = iClose(Symbol1, PERIOD_H1, i + 1);
				double y = iClose(Symbol2, PERIOD_H1, i + 1);

				sum_x += x;
				sum_y += y;
				sum_xy += (x * y);
				sum_xx += (x * x);
				sum_yy += (y * y);
			}
		//DeBugEnd();
		return	(
					(eiLookBackPeriod*sum_xy-sum_x*sum_y)/
					MathSqrt(	(eiLookBackPeriod*sum_xx -(sum_x*sum_x))
							*	(eiLookBackPeriod*sum_yy -(sum_y*sum_y)))
				);
	}

//+-------------------------------------------------------------------------------------------------+
//| Filter allowed trade pairs based on selected swap mode											|
//+-------------------------------------------------------------------------------------------------+
void FilterBySwap(int i=0) 
	{
		if (eiSwapMode	==	SWAPDONTCARE)
			return;
	//DeBug("FilterBySwap");
		double swap;
		//--- handle buys ---
		for(i=iPairBuy-1;i	>=	0;i--) 
			{
				swap = MarketInfo(saPairBuy[i], MODE_SWAPLONG);
				if((eiSwapMode	==	SWAPBETTER		&&	swap	<MarketInfo(saPairBuy[i],MODE_SWAPSHORT))
				|| (eiSwapMode	==	SWAPPOSITIVE	&&	swap	<0))
					removeElementID(saPairBuy,i,iPairBuy);
			}
		//--- handle sells ---
		for(i=iPairSell-1;i	>=	0;i--) 
			{
				swap = MarketInfo(saPairSell[i], MODE_SWAPSHORT);
				if((eiSwapMode	==	SWAPBETTER		&&	swap<MarketInfo(saPairSell[i],MODE_SWAPLONG))
				|| (eiSwapMode	==	SWAPPOSITIVE	&&	swap<0))
					removeElementID(saPairSell,i,iPairSell);
			}
	//DeBugEnd();
	}

//+-------------------------------------------------------------------------------------------------+
//| pause time to string																			|
//+-------------------------------------------------------------------------------------------------+
string PauseToStr(int ptime) 
	{
	//DeBug("PauseToStr");
		string	dayname[7]	=	{"Sun","Mon","Tue","Wed","Thu","Fri","Sat"};
		string	hour	=	padLeft((ptime/100)%100,2);
		string 	minute	=	padLeft(ptime%100,2);
		return(dayname[ptime/10000]+" "+hour+":"+minute);
	//DeBugEnd();
	}
	
string padLeft(string stringIn, int length, string pad	=	"0")
	{
		while(StringLen(stringIn) < length)
			stringIn	=	pad	+	stringIn;
		return(stringIn);
	}

//+-------------------------------------------------------------------------------------------------+
//|	Arrange currencies	into original xMeter Indicator order										|																							|
//+-------------------------------------------------------------------------------------------------+
int seq(int i)
	{
		int   o[]={   6,3,4,2,1,0,5   };
		return(o[i]);
	}

void setUpNewTrades(int i=0)
	{
	//DeBug("setUpNewTrades");
		//--- iterate over all allowed trade pairs in their priority ---
		for(;i	<	MathMax(iPairBuy,iPairSell);i++) 
			{
				if (i	<	iPairBuy)
					{
						//--- check buying of PairID pair ---
						//--- get currency IDs ---
						// do not use currency with sparse data
						//--- if xMeter reaches edTradeTrigger, enter the trade ---
						sPairType	=	saPairBuy[i];
						iDigit		=	MarketInfo(sPairType,MODE_DIGITS);
						iBase		=	binarySearch(saCurrency,StringSubstr(sPairType, 0, 3));
						iQuote		=	binarySearch(saCurrency,StringSubstr(sPairType, 3, 3));
						if (MathMin(iaPairCount[iBase],iaPairCount[iQuote])	>=	MINIMUMCURRENCYPAIRS
						&&	MathMin(daCurrencyMeter[iBase],daCurrencyMeter[iQuote])	>	0
						&&	daCurrencyMeter[iBase]-daCurrencyMeter[iQuote]	>=	edTradeTrigger)
							if (manageBuy(	sPairType,iDigit,"["+DoubleToStr(daCurrencyMeter[iBase],1)	+
														"|"+DoubleToStr(daCurrencyMeter[iQuote],1)+"]"))
								checkTradeList(true);
					}
				if (i	<	iPairSell)
					{
						//--- check buying of PairID pair ---
						//--- get currency IDs ---
						// do not use currency with sparse data
						//--- if xMeter reaches edTradeTrigger then enter the trade
						sPairType	=	saPairSell[i];
						iDigit		=	MarketInfo(sPairType,MODE_DIGITS);
						iBase		=	binarySearch(saCurrency,StringSubstr(sPairType, 0, 3));
						iQuote		=	binarySearch(saCurrency,StringSubstr(sPairType, 3, 3));
						if (MathMin(iaPairCount[iBase],iaPairCount[iQuote])	>=	MINIMUMCURRENCYPAIRS
						&&	MathMin(daCurrencyMeter[iBase],daCurrencyMeter[iQuote])	>	0
						&&	daCurrencyMeter[iQuote]-daCurrencyMeter[iBase]	>=	edTradeTrigger)
							if (manageSell(	sPairType,iDigit,	"["+DoubleToStr(daCurrencyMeter[iBase],1)+
																"|"+DoubleToStr(daCurrencyMeter[iQuote],1)+"]"))
								checkTradeList(true);
					}
			}
	//DeBugEnd();
	}

void checkTradeList(bool correlationFilter)
	{
		if (ebAutoSelectPairs)
			selectTradePairs();
		else
			if (correlationFilter)
				filterCorrelatedPairs();
		FilterBySwap();
	}

void processOpenClosePositions()
	{
	//DeBug("processOpenClosePositions");
		//--- open recovery positions if necessary or close open ones ---
		int	i,j	=	ArraySize(saPairType);
		for(;i	<	j;i++) 
			{
				//--- extract currencies ---
				sPairType	=	StringSubstr(saPairType[i],0,6);
				iDigit		=	MarketInfo(sPairType,MODE_DIGITS);
				iBase		=	binarySearch(saCurrency,StringSubstr(sPairType, 0, 3));
				iQuote		=	binarySearch(saCurrency,StringSubstr(sPairType, 3, 3));
				//--- pair might be traded, check trading conditions --
				//--- if xMeter falls to RecoverTrigger,allow another trade in same currency pair
				if (MathMin(daTriggerMeter[iBase],daTriggerMeter[iQuote])	>	0) 
					{
						if (daTriggerMeter[iBase]-daTriggerMeter[iQuote]	<=	edRecoverTrigger
						&&	GlobalVariableGet(sRoot+sPairType+"buy")	==	2)
							manageBuy(	sPairType,
										iDigit,	"["+DoubleToStr(daTriggerMeter[iBase],1)+
												"|"+DoubleToStr(daTriggerMeter[iQuote],1)+"]");
	
						if (daCurrencyMeter[iBase]-daCurrencyMeter[iQuote]	<=	edRecoverTrigger
						&&	GlobalVariableGet(sRoot+sPairType+"sell")	==	2)
							manageSell(	sPairType,
										iDigit,	"["+DoubleToStr(daCurrencyMeter[iBase],1)+
												"|"+DoubleToStr(daCurrencyMeter[iQuote],1)+"]");
					}
			}
	//DeBugEnd();
	}
	
void calculateAllCurrencies()
	{
	//DeBug("calculateAllCurrencies");
		//---- calculate all currencies meter
		for(int i=0;i	<	iCurrency; i++) 
			if (iaDivisor[i]!=0) 
				{
					daTriggerMeter[i]=0;
					for(int j=0;j	<	iPair; j++) 
						if (StringSubstr(saPair[j],0,3)==saCurrency[i])
							daTriggerMeter[i]+=daTriggerMajor[j];
						else
							if (StringSubstr(saPair[j],3,3)==saCurrency[i])
								daTriggerMeter[i]+=daTriggerMinor[j];
					daTriggerMeter[i]=NormalizeDouble(daTriggerMeter[i]/iaDivisor[i],1);
					daCurrencyMeter[i]=0;
					for(j=0; j!=iPair; j++)
						if (StringSubstr(saPair[j],0,3)==saCurrency[i])
							daCurrencyMeter[i]+=daCurrencyMajor[j];
						else
							if (StringSubstr(saPair[j],3,3)==saCurrency[i])
								daCurrencyMeter[i]+=daCurrencyMinor[j];
					daCurrencyMeter[i]=NormalizeDouble(daCurrencyMeter[i]/iaDivisor[i],1);
				}
	//DeBugEnd();
	}

void calculateDivisors()
	{
	//DeBug("calculateDivisors");
		//---- calculate divisors for averaging
		ArrayInitialize(iaDivisor,0);
		ArrayInitialize(iaPairCount,0);
		for(int i=0;i	<	iCurrency; i++) 
			for(int j=0;j	<	iPair; j++)
				if (StringSubstr(saPair[j],0,3)==saCurrency[i]) 
					{
						iaPairCount[i]++;
						if (daTriggerMajor[j]>0)
							iaDivisor[i]++;
					}
				else
					if (StringSubstr(saPair[j],3,3)==saCurrency[i]) 
						{
							iaPairCount[i]++;
							if (daTriggerMinor[j]>0)
									iaDivisor[i]++;
						}
	//DeBugEnd();
	}

void calculateMeter()
	{
	//DeBug("calculateMeter");
		//--- !xMeter calculation ---
		int	i;
		for(;i	<	iPair;i++) 
			{
				RefreshRates();		// refresh all currencies
				string	lsPair	=	saPair[i]+esSuffix;
				double	ask		=	MarketInfo(lsPair, MODE_ASK);
				double	bid		=	MarketInfo(lsPair, MODE_BID);
				double	point	=	MarketInfo(lsPair, MODE_POINT);
				int		sprd	=	MarketInfo(lsPair, MODE_SPREAD);
				int		digit	=	MarketInfo(lsPair, MODE_DIGITS);

				if (digit	==	3	||	digit	==	5) 
					point	*=	10;
					
				// Get point info for symbol for shorter lookback period &entry confirmation & exit 
				// Find hi bar & lo bar for last 1/3 of lookback period and set low and close

				int daTriggerhighBar=	iHighest(lsPair,60,MODE_HIGH, 	eiLookBackPeriod / 3, 0);
				int daTriggerlowBar	=	iLowest	(lsPair,60,MODE_LOW,	eiLookBackPeriod / 3, 0);
				daTriggerHigh[i]	=	iHigh	(lsPair,60,daTriggerhighBar);	//--- Set high
				daTriggerLow[i]		=	iLow	(lsPair,60,daTriggerlowBar);								
				daTriggerAsk[i]		=	iClose	(lsPair,60,0);										
				daTriggerRange[i]	=	MathMax	((daTriggerHigh[i]-daTriggerLow[i])/point,1);
				daTriggerRatio[i]	=	(daTriggerAsk[i]-daTriggerLow[i])/daTriggerRange[i]/point;		

				//--- Calculate pair's ratio and inverse ratio
				int daCurrencyhighBar=	iHighest(lsPair,60,MODE_HIGH,	eiLookBackPeriod, 0);
				int daCurrencylowBar=	iLowest	(lsPair,60,MODE_LOW,	eiLookBackPeriod, 0);
				daCurrencyHigh[i]	=	iHigh	(lsPair,60,daCurrencyhighBar);
				daCurrencyLow[i]	=	iLow	(lsPair,60,daCurrencylowBar);
				daCurrencyAsk[i]	=	iClose	(lsPair,60,0);
				daCurrencyRange[i]	=	MathMax	((daCurrencyHigh[i]-daCurrencyLow[i])/point,1);
				daCurrencyRatio[i]	=	(daCurrencyAsk[i]-daCurrencyLow[i])	/daCurrencyRange[i]
																				/point;
				daCurrencyWeight[i]	=	(daCurrencyRatio[i] *2+daTriggerRatio[i])/3;
				// Grade table for entry confirmation & exit trigger
				daTriggerMajor[i]	=	calculatedScore(daTriggerRatio[i]);	//-Set pair grade
				daTriggerMinor[i]	=	9-daTriggerMajor[i]; 				//-Set inverse grade

				// Grade table for currency's power
				daCurrencyMajor[i]	=	calculatedScore(daCurrencyWeight[i]);	//-Set pair grade 
				daCurrencyMinor[i]	=	9-daCurrencyMajor[i]; 					//-Set inverse

				double	LowestBuy	=	1000,
						LowestSell	=	1000,
						BuyProfits	=	0,
						SellProfits	=	0;
						
				int		ix			=	0,
						Orders		=	0,
						SellOrders	=	0,
						BuyOrders	=	0; 

				//--- calculate buy orders ---
				ix	=	serialSearch(saPairType,lsPair+"(+)",iPairType);
				if (ix	!=	-1) 
					{
						Orders		+=	iaCount[ix];
						BuyOrders	=	iaCount[ix];
						BuyProfits	=	daProfit[ix];
					}
				//--- calculate sell orders ---
				ix	=	serialSearch(saPairType,lsPair+"(-)",iPairType);
				if (ix	!=	-1) 
					{
						Orders		+=	iaCount[ix];
						SellOrders	=	iaCount[ix];
						SellProfits	=	daProfit[ix];
					}
				//--- check recovery close (if TP couldn't be set - e.g. order too close to market 
				//--- or price within freeze level, etc.) ---
				if (BuyOrders	==	0)
					GlobalVariableSet(sRoot+lsPair+"buy",1);
				if (SellOrders	==	0)
					GlobalVariableSet(sRoot+lsPair+"sell",1);

				//--- adjust TP and SL ---
				dynamicBuyTP(	lsPair,
								BuyOrders,
								point,
								sprd,
								digit,		// Number of digits
								daPrice[serialSearch(saPairType,sSymbol+"(+)",iPairType)]);
				dynamicSellTP(	lsPair,
								SellOrders,
								point,
								sprd,
								digit,		// Number of digits
								daLots[serialSearch(saPairType,sSymbol+"(-)",iPairType)]);
				dynamicBuySL(	lsPair,
								BuyOrders,
								bid,
								point,
								sprd,
								digit);		// Number of digits
				dynamicSellSL(	lsPair,
								SellOrders,
								ask,
								point,
								sprd,
								digit);		// Number of digits
			}
	//DeBugEnd();
	}
	
void calculateAveragePrices()
	{
	//DeBug("calculateAveragePrices");
		//--- calculate average prices(accumulated swap and commission values modify average price!)
		int	i,j	=	ArraySize(saPairType);
		for(;i	<	j;i++) 
			{
				double tickval	=	MarketInfo(StringSubstr(saPairType[i],0,
															StringLen(saPairType[i])-3),
															MODE_TICKVALUE)*daLots[i];
				double pnt		=	MarketInfo(StringSubstr(saPairType[i],0,
															StringLen(saPairType[i])-3),
															MODE_POINT);
				//--- weighted average of open prices (weight = lot size) ---
				//DeBug("Pair "+saPairType[i]+",Price: "+daPrice[i]+",Lots: "+daLots[i]);DeBugEnd();
				daPrice[i]=daPrice[i]/daLots[i];
				//--- modify average based on swap & commission ---
				if (StringSubstr(saPairType[i],StringLen(saPairType[i])-3,3)=="(+)")
					//--- buy ---
					daPrice[i]-=(daFees[i]/tickval)*pnt;
				else
					//--- sell ---
					daPrice[i]+=(daFees[i]/tickval)*pnt;
			}
	//DeBugEnd();
	}

//--- collect active pairs ---		
void collectActivePairs()
	{
		iBuy	=	0;
		iSell	=	0;	
	
		ArrayResize(saBuy,0);
		ArrayResize(saSell,0);
		int		i,j	=	OrdersTotal();
		for(;i	<	j;i++) 
			if (OrderSelect(i,SELECT_BY_POS)
			&&	OrderMagicNumber()	==	eiMagicNo) 
				if (OrderType()	==	OP_BUY)
					addToArray(saBuy,OrderSymbol(),iBuy);
				else
					if (OrderType()==OP_SELL) 
						addToArray(saSell,OrderSymbol(),iSell);
	}

//+-------------------------------------------------------------------------------------------------+
//| Select uncorrelated pairs and order them by current spread										|
//+-------------------------------------------------------------------------------------------------+
void selectTradePairs() 
	{
	//DeBug("selectTradePairs");
		int	i;
		collectActivePairs();
		iCount	=	0;
		ArrayResize(saSymbols,iCount);
		//--- query all symbols and their spreads ---
//=========================================================
 		for(;i	<	iPair;i++) 
			{
				iBase	=	binarySearch(saCurrency,StringSubstr(saPair[i], 0, 3));
				iQuote	=	binarySearch(saCurrency,StringSubstr(saPair[i], 3, 3));
				// do not use currency with sparse data
				if (MathMin(iaPairCount[iBase],iaPairCount[iQuote])	>=	MINIMUMCURRENCYPAIRS)
					addToArray(saSymbols,DoubleToStr(	1000+MarketInfo(saPair[i]+esSuffix,MODE_SPREAD),0)+
														saPair[i]+esSuffix,
														iCount);
			}
//===========================================================
		quickSort(saSymbols,0,iCount-1);
		for(i=0;i	<	iCount;i++) // remove spread prefix
			saSymbols[i]=StringSubstr(saSymbols[i],4);
		//--- allow everything at first ---
		iPairBuy	=	iPair;
		iPairSell	=	iPair;
		ArrayResize(saPairBuy,iPairBuy);
		ArrayResize(saPairSell,iPairSell);
		ArrayCopy(saPairBuy,saPair);
		ArrayCopy(saPairSell,saPair);
		calculateCorrelations();
	}
	
void calculateCorrelations()
	{
		//--- calculate correlations for existing pairs ---
		//         correlation >= 0.75   ->  only reverse direction allowed
		//         correlation <= -0.75  ->  only same direction allowed
		// -0.75 < correlation <  0.75   ->  any entry allowed
		int		i,j,ix;
		string	pair1,
				pair2;
		double	correl;
		ArrayResize(saCorrel,iPairType);
		for(;i	<	iPairType;i++)
			saCorrel[i]="";
		for(i=iCount-1;i	>=	0;i--) 
			{
				pair1	=	saSymbols[i];
				for(j=0;j	<	iBuy;j++) 
					{
						pair2=saBuy[j];
						//--- find pair iIndex ---
						ix		=	serialSearch(saPairType,pair2+"(+)",iPairType);
						//calculate correlation between pair1 and pair2
						correl	=	Correlation(pair1,pair2);
						if (correl	>=	CORRELATION) 
							{
								//--- existing pair1 and pair2 (existing buy) correlate
								//--- in same direction --- deny buying pair1 ---
								removeElement(saPairBuy,pair1,iPairBuy);
								if (pairsRelated(pair1,pair2))
                  					if (saCorrel[ix]		==	"")
                     					saCorrel[ix]		=	StringSubstr(pair1, 0, 6) + "(+)";
               						else
                  						saCorrel[ix]		=	saCorrel[ix]+"; "+
																StringSubstr(pair1,0,6)+"(+)";
              				}
						else
							if (correl	<=	-CORRELATION) 
								{
									//--- existing pair1 and pair2 (existing buy) correlate
									//--- in same direction --- deny selling pair1 ---
									removeElement(saPairSell,pair1,iPairSell);
 									if (pairsRelated(pair1,pair2))
										if (saCorrel[ix]		==	"")
											saCorrel[ix]		=	StringSubstr(pair1, 0, 6) + "(-)";
										else
											saCorrel[ix]		=	saCorrel[ix]+"; "+
																	StringSubstr(pair1,0,6)+"(-)";
								}
					}
				for(j=0;j	<	iSell;j++) 
					{
						pair2	=	saSell[j];
						//--- find pair iIndex ---
						ix		=	serialSearch(saPairType,pair2+"(-)",iPairType);
						//calculate correlation between pair1 and pair2
						correl	=	Correlation(pair1,pair2);
						if (correl	>=	CORRELATION) 
							{
								//--- existing pair1 and pair2 (existing sell) correlate
								//--- in same direction --- deny selling pair1 ---
								removeElement(saPairSell,pair1,iPairSell);
								if (pairsRelated(pair1,pair2))
									if (saCorrel[ix]		==	"")
										saCorrel[ix]		=	StringSubstr(pair1, 0, 6) + "(+)";
									else
										saCorrel[ix]		=	saCorrel[ix]+"; "+
																StringSubstr(pair1,0,6)+"(+)";
							}
						else
							if (correl	<=	-CORRELATION) 
								{
									//--- existing pair1 and pair2 (existing sell) correlate
									//--- in same direction --- deny buying pair1 ---
									removeElement(saPairBuy,pair1,iPairBuy);
									if (pairsRelated(pair1,pair2))
										if (saCorrel[ix]	==	"")
											saCorrel[ix]	=	StringSubstr(pair1,0,6)+"(-)";
										else
											saCorrel[ix]	=	saCorrel[ix]+"; "+
																	StringSubstr(pair1,0,6)+"(-)";
								}
					}
			}
		//--- remove symbols that cannot be bought or sold ---
		for(i=iCount-1;i	>=	0;i--) 
			if (serialSearch(saPairBuy,saSymbols[i],iPairBuy)	<	0
			&&	serialSearch(saPairSell,saSymbols[i],iPairSell)	<	0)
				removeElementID(saSymbols,i,iCount);
	//DeBugEnd();	
	}

bool pairsRelated(string spPair1, string spPair2)
	{
		//return (spPair1	!=	spPair2);
		return (spPair1	!=	spPair2	
				&& (StringFind(spPair1,StringSubstr(spPair2,0,3))	>=	0
				||	StringFind(spPair1,StringSubstr(spPair2,3,3))	>=	0));
	}

//+-------------------------------------------------------------------------------------------------+
//| Select uncorrelated pairs                                        								|
//+-------------------------------------------------------------------------------------------------+
void filterCorrelatedPairs() 
	{
	//DeBug("filterCorrelatedPairs");
		int	i,j;
		collectActivePairs();
		iCount	=	0;
		ArrayResize(saSymbols,iCount);
		//--- process list of pairs allowed to trade and order them by spread ---
		string list	=	esTradePairs+",";
		j	=	StringFind(list,",");
		while(StringLen(list)	>	1) 
			{
				string	pair	=	StringTrimLeft(StringTrimRight(StringSubstr(list,0,j)));
				int		ask		=	MarketInfo(pair+esSuffix,MODE_ASK);
				//if broker does not offer currency pair then exclude from xMeter averaging
				if (ask	==	0) 
					Alert(	"Unknown pair to trade: ",pair+esSuffix,", pair ignored."+NL+
							"Check if broker provides pair and it is visible in Market Watch.");
				else 
					addToArray(saSymbols,DoubleToStr(	1000+MarketInfo(pair+esSuffix,MODE_SPREAD),0)+
														pair+esSuffix,
														iCount);
				list	=	StringSubstr(list,j+1);
			}

		quickSort(saSymbols,0,iCount-1);
		for(;i	<	iCount;i++) // remove spread prefix
			saSymbols[i]	=	StringSubstr(saSymbols[i],4);
		//--- allow everything at first ---
		iPairBuy	=	iCount;
		iPairSell	=	iCount;
		ArrayResize(saPairBuy,iPairBuy);
		ArrayResize(saPairSell,iPairSell);
		ArrayCopy(saPairBuy,saSymbols);
		ArrayCopy(saPairSell,saSymbols);
		calculateCorrelations();
	//DeBugEnd(); 
	}

//+-------------------------------------------------------------------------------------------------+
//| collect Existing Trades                                       	 								|
//+-------------------------------------------------------------------------------------------------+
void collectExistingTrades()
	{
	//DeBug("collectExistingTrades");
		//--- collect existing trade Data ---
		int	i,j	=	resizeOrderArrays(OrdersTotal());
		for(;i	<	j; i++) 
			if (OrderSelect(i,SELECT_BY_POS)
			&&	OrderMagicNumber()	==	eiMagicNo
			&&	OrderType()			<=	OP_SELL)
				{
					sPairType		=	OrderSymbol()+saOrderType[OrderType()];
					int	ix			=	serialSearch(saPairType,sPairType,iPairType);
					if (ix	<	0)
						addToOrders(ix);
					iaCount[ix]		++;
					daPrice[ix]		+=	OrderOpenPrice()*OrderLots();
					daLots[ix]		+=	OrderLots();
					daProfit[ix]	+=	OrderProfit()+OrderSwap()+OrderCommission();
					daFees[ix]		+=	OrderSwap()+OrderCommission();
				}
	}

void addToOrders(int& i)
	{
		//--- add symbol ---
		i	=	iPairType;
		addToArray(saPairType,sPairType,iPairType,iPairType);	// add at the End
		ArrayResize(saPairType,iPairType);
		ArrayResize(daPrice,iPairType);
		ArrayResize(daLots,iPairType);
		ArrayResize(daProfit,iPairType);
		ArrayResize(iaCount,iPairType);
		ArrayResize(daFees,iPairType);

		iaCount[i]	=	0;
		daPrice[i]	=	0;
		daLots[i]	=	0;
		daProfit[i]	=	0;
		daFees[i]	=	0;
	}

void checkForMultipleExecutions()
	{
	//DeBug("checkForMultipleExecutions");
		//alert if EA running on another chart
		if (GlobalVariableGet(sRoot+".Initialized")	!=	iCycle) 
			{
				Alert("!xMeter EA Conflicts... Restarting...");
				init();
			}
	//DeBugEnd();
	}

void allocateArrays()
	{
	//DeBug("allocateArrays");
		//--- allocate other arrays ---
		ArrayResize(daTriggerMeter,iCurrency);
		ArrayResize(daTriggerHigh,iPair);
		ArrayResize(daTriggerLow,iPair);
		ArrayResize(daTriggerAsk,iPair);
		ArrayResize(daTriggerRatio,iPair);
		ArrayResize(daTriggerRange,iPair);
		ArrayResize(daTriggerMajor,iPair);
		ArrayResize(daTriggerMinor,iPair);
		
		ArrayResize(daCurrencyMeter,iCurrency);
		ArrayResize(daCurrencyHigh,iPair);
		ArrayResize(daCurrencyLow,iPair);
		ArrayResize(daCurrencyAsk,iPair);
		ArrayResize(daCurrencyRatio,iPair);
		ArrayResize(daCurrencyRange,iPair);
		ArrayResize(daCurrencyMajor,iPair);
		ArrayResize(daCurrencyMinor,iPair);
		ArrayResize(daCurrencyWeight,iPair);
		
		ArrayResize(iaDivisor,iCurrency);
		ArrayResize(iaPairCount,iCurrency);
	//DeBugEnd();
	}

void constructCurrencies()
	{
	//DeBug("constructCurrencies");
		for(int i=0; i<iPair; i++) 
			{
				addToArray(saCurrency,StringSubstr(saPair[i],0,3),iCurrency);
				addToArray(saCurrency,StringSubstr(saPair[i],3,3),iCurrency);
			}
		ArrayResize(saCurrency,iCurrency);			
		quickSort(saCurrency,0,iCurrency-1);
	//DeBugEnd();
	}
	
void constructBrokerPairs(string pCurrency[])
	{
	//DeBug("constructBrokerPairs");
		int iCurrencies	=	ArraySize(pCurrency);
		for(int i=0;i	<	iCurrencies;i++)
			for(int j=0;j	<	iCurrencies;j++) 
				if (i	!=	j)
					addToPairs(pCurrency[i]	+	pCurrency[j]);					
		string	sPair,
				sSpecialPairs	=	esSpecialPairs+",";
		while(StringLen(sSpecialPairs)	>	1)
			{
				sPair			=	StringSubstr(sSpecialPairs,0,StringFind(",",sSpecialPairs));
				sSpecialPairs	=	StringSubstr(sSpecialPairs,StringLen(sPair)+1);
				if (!addToPairs(sPair))
					Alert(	"Unknown special pair: ",sPair+esSuffix,", pair ignored."+NL+
							"Check if broker provides symbol and it is visible in Market Watch.");
			}
		ArrayResize(saPair,iPair);			
		quickSort(saPair,0,iPair-1);
	//DeBugEnd();
	}

bool addToPairs(string pElement)
	{
		if (MarketInfo(pElement+esSuffix,MODE_ASK)	==	0)
			return(false);
		//DeBug("addToPairs");
		addToArray(saPair,pElement,iPair);
		//DeBugEnd();
		return(true);
	}

void checkAlert()
	{
		int	i,j	=	OrdersTotal();
		for(;i	<	j;i++) 
			if (OrderSelect(i,SELECT_BY_POS,MODE_TRADES)
			&&	OrderMagicNumber()	==	eiMagicNo) 
				{
					bAlertNow	=	true;
					tLastErrorTime	=	TimeCurrent();
				}
	}

void addToArray(string& pArray[], string pElement, int& pCount, int i=0)
	{
		for(;i	<	pCount;i++)
			if (pElement	==	pArray[i])
				return;
	//DeBug("addToArray");
		pCount++;
		ArrayResize(pArray,pCount);
		pArray[i]	=	pElement;
	//DeBug("addToArray: Element# " +padLeft(pCount,2)+"."+pElement+" Added");DeBugEnd();
	//DeBugEnd();
	}

//+------------------------------------------------------------------+
//| remove Element from array                                        |
//+------------------------------------------------------------------+
bool removeElement(string& pArray[], string pElement, int& pCount, int i=0) 
	{
		for(;i	<	pCount;i++)
			if (pArray[i]	==	pElement) 
				return(removeElementID(pArray,i,pCount));
	//DeBug(pElement +" Element Not Found");DeBugEnd();
		return(false);
	}

//+-------------------------------------------------------------------------------------------------+
//| remove Element from Array by Location															|
//+-------------------------------------------------------------------------------------------------+
bool removeElementID(string &pArray[], int i, int &pCount)
	{
		if (StringLen(pArray[i])	<	1
		||	i	<	0	||	i	>	pCount	-1)
			return(false);
		for(;i	<	pCount-1;i++)
			pArray[i]	=	pArray[i+1];
		pCount--;
		ArrayResize(pArray,pCount);
		return(true);
	}

//+-----------------------------------------------------------------------------+
//| Binary Search for an Element in an Array									|
//+-----------------------------------------------------------------------------+
int binarySearch(string& pArray[], string pElement)
	{
		int	iLo	=	0,
			iMd	=	0,
			iHi	=	ArraySize(pArray)	-	1;
		while(iLo	<=	iHi)
			{
				iMd	=	(iLo	+	iHi)/2;
				if (pElement	==	pArray[iMd])
					return(iMd);
				if (pElement	<	pArray[iMd])
					iHi	=	iMd	-	1;
				else
					iLo	=	iMd	+	1;
			}
		return(-1);
	}

int serialSearch(string& pArray[], string pElement, int pCount, int i=0)
	{
		for(;i<pCount;i++)
			if (pArray[i]	==	pElement)
				return(i);
		return(-1);
	}
	
int resizeOrderArrays(int j)
	{
		iPairType	=	0;
		ArrayResize(saPairType,0);
		ArrayResize(daPrice,0);
		ArrayResize(daLots,0);
		ArrayResize(daProfit,0);
		ArrayResize(iaCount,0);
		ArrayResize(daFees,0);
		return(j);
	}

string leftPad(string s, int l, string p="0")
	{	
		while(StringLen(s) < l)
			s	=	p+s;
		return(s);
	}


